// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "GridBenchmark.h"
#include "MainFrm.h"
#include "AddItemsFromNonGUIThreadJob.h"
#include "RemoveItemsFromNonGUIThreadJob.h"
#include "AddItemsDlg.h"
#include "RemoveItemsDlg.h"
#include "UpdateItemsDlg.h"
#include "TestObject.h"
#include "LogMessage.h"

#include <Dapfor/GUI/Header.h>
#include <Dapfor/GUI/SortInfo.h>
#include <Dapfor/GUI/MsImg32.h>

#include <Dapfor/Common/Lock.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CGridBenchmarkApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
    //{{AFX_MSG_MAP(CMainFrame)
    ON_WM_CREATE()
    ON_WM_SETFOCUS()
    ON_COMMAND(ID_ADD_ITEMS, OnAddItems)
    ON_COMMAND(ID_REMOVE_ITEMS, OnRemoveItems)
    ON_COMMAND(ID_NOTIFICATIONS_OPTIMIZATION, OnNotificationsOptimization)
    ON_UPDATE_COMMAND_UI(ID_NOTIFICATIONS_OPTIMIZATION, OnUpdateNotificationsOptimization)
    ON_COMMAND(ID_ONIDLE_OPTIMIZATION, OnOnidleOptimization)
    ON_UPDATE_COMMAND_UI(ID_ONIDLE_OPTIMIZATION, OnUpdateOnidleOptimization)
    ON_WM_ERASEBKGND()
    ON_WM_DESTROY()
    ON_WM_TIMER()
    ON_COMMAND(ID_UPDATE_SETTINGS, OnUpdateSettings)
    ON_COMMAND(ID_ENABLE_UPDATES, OnEnableUpdates)
    ON_UPDATE_COMMAND_UI(ID_ENABLE_UPDATES, OnUpdateEnableUpdates)
    ON_COMMAND(ID_HIGHLIGHT_ALPHA, OnHighlightAlpha)
    ON_UPDATE_COMMAND_UI(ID_HIGHLIGHT_ALPHA, OnUpdateHighlightAlpha)
    ON_COMMAND(ID_HIGHLIGHT_DISABLE, OnHighlightDisable)
    ON_UPDATE_COMMAND_UI(ID_HIGHLIGHT_DISABLE, OnUpdateHighlightDisable)
    ON_COMMAND(ID_HIGHLIGHT_FADING, OnHighlightFading)
    ON_UPDATE_COMMAND_UI(ID_HIGHLIGHT_FADING, OnUpdateHighlightFading)
    ON_COMMAND(ID_DOUBLE_BUFFERING, OnDoubleBuffering)
    ON_UPDATE_COMMAND_UI(ID_DOUBLE_BUFFERING, OnUpdateDoubleBuffering)
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
 : m_EnableUpdates(true)
 , m_UpdatesSource(GUIThread)
 , m_Frequency(0)
 , m_UpdatedItems(0)
 , m_StartTime(GetTickCount())
 , m_TotalUpdatedItems(0)
 , m_LastCPUUsageTick(0)
 , m_LastConsumedTime(0)
{
    
}

CMainFrame::~CMainFrame()
{
    Common::CLock lock(m_Synchro);

    //Delete all test objects
    {
        for(Container::iterator it = m_TestObjects.begin(); it != m_TestObjects.end(); ++it)
        {
            delete *it;
        }
    }


    //Delete all log messages
    {
        for(LogMessages::iterator it = m_LogMessages.begin(); it != m_LogMessages.end(); ++it)
        {
            delete *it;
        }
    }

}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
        return -1;

    //Create splitter window
    if (!m_wndSplitter.CreateStatic(this, 2, 1)) return FALSE;
    m_wndSplitter.SetRowInfo( 0, 350, 0 );
    m_wndSplitter.SetRowInfo( 1, 60, 0 );

    m_Grid.Create(0, 0, WS_VISIBLE|WS_CHILD, CRect(), &m_wndSplitter, m_wndSplitter.IdFromRowCol(0, 0));
    GUI::CHeader* header = new GUI::CHeader(true);
    header->Add(new GUI::CColumn(CTestObject::FidName, _T("Name"),     70));
    header->Add(new GUI::CColumn(CTestObject::Fid1,    _T("Column1"),  80));
    header->Add(new GUI::CColumn(CTestObject::Fid2,    _T("Column2"),  80));
    header->Add(new GUI::CColumn(CTestObject::Fid3,    _T("Column3"),  80));
    header->Add(new GUI::CColumn(CTestObject::Fid4,    _T("Column4"),  80));
    header->Add(new GUI::CColumn(CTestObject::Fid5,    _T("Column5"),  80));
    header->Add(new GUI::CColumn(CTestObject::Fid6,    _T("Column6"),  80));
    header->Add(new GUI::CColumn(CTestObject::Fid7,    _T("Column7"),  80));
    header->Add(new GUI::CColumn(CTestObject::Fid8,    _T("Column8"),  80));
    header->Add(new GUI::CColumn(CTestObject::Fid9,    _T("Column9"),  80));
    header->Add(new GUI::CColumn(CTestObject::Fid10,   _T("Column10"), 80));
    m_Grid.SetHeader(header);
    m_Grid.AllowNotifications(false);
    m_Grid.GetHighlightSettings().SetMode(GUI::CHighlightSettings::AlphaBlendBackColor);
	m_Grid.GetHighlightSettings().SetBackColor(RGB(230, 10, 14));
	m_Grid.AllowHighlight(true);


    
    //Remove the place reserved to paint expansion icon in the hierarchical grid
    UINT filter = m_Grid.GetPaintFilter();
    filter &= filter^GUI::drawHierarchy;
    m_Grid.SetPaintFilter(filter);
	m_Grid.GetAppearance().SetBackOddColor(RGB(230,237,237));


    //Create logger
    m_Logger.Create(0, 0, WS_VISIBLE|WS_CHILD, CRect(), &m_wndSplitter, m_wndSplitter.IdFromRowCol(1, 0));
    //Create a header for the logger. True in the constructor indicates, 
    //that the header will be destroyed by the grid in its destructor.
    header = new GUI::CHeader(true);
    header->Add(new GUI::CColumn(CLogMessage::FidTimestamp,  _T("Time"), 70));
    header->Add(new GUI::CColumn(CLogMessage::FidActionType, _T("Action"), 50, GUI::CColumn::Left));
    header->Add(new GUI::CColumn(CLogMessage::FidMessage, _T("Message"), 700));

    m_Logger.SetHeader(header);
    m_Logger.GetAppearance().SetBackOddColor(RGB(240, 240, 240));
    m_Logger.SetSort(0, GUI::CSortInfo(CLogMessage::FidTimestamp, true), true);
    
    //Remove the place reserved to paint expansion icon in the hierarchical grid
    filter = m_Logger.GetPaintFilter();
    filter &= filter^GUI::drawHierarchy;
    m_Logger.SetPaintFilter(filter);
    m_Logger.AllowHighlight(false);
    
    
    //Load toolbar
    if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
        | CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
        !m_wndToolBar.LoadToolBar(IDR_TOOLBAR))
    {
        TRACE0("Failed to create toolbar\n");
        return -1;      // fail to create
    }
    
    //Set 32-bit color icons
    GUI::CMsImg32::Load32BitImage(m_ToolbarIcons, IDB_TOOLBAR, true, RGB(0, 0, 0));
    m_wndToolBar.GetToolBarCtrl().SetImageList(&m_ToolbarIcons);
    

    //Start non-GUI thread
    m_Thread.Start();
    
    SetTimer(TimerId, 10, 0);               //Start GUI timer to process item updates from the GUI thread.
    SetTimer(TitleUpdateTimerId, 3000, 0);  //This timer measures the CPU usage and updates the title
    Common::CTimer::StartTimer(this, TimerId, 10); //Start timer to process item updates from the non-GUI thread.

    //Initialize CPU usage counter
    m_LastCPUUsageTick = GetTickCount();

    //Update the title
    UpdateTitle();

#ifndef DEBUG
	//In the release version add some items to the grid
	AddItemsToGrid(3000, Tail, GUIThread);
	m_UpdatedItems = 0;
	m_Frequency = 5000;
	m_StartTime = GetTickCount();
#endif

    return 0;
}


BOOL CMainFrame::OnEraseBkgnd(CDC* pDC) 
{
    //Prevent from flickering
    return TRUE;
}

void CMainFrame::OnDestroy() 
{
    KillTimer(TitleUpdateTimerId);
    KillTimer(TimerId);
    Common::CTimer::StopTimer(this, TimerId);

    m_Thread.Stop();        //Stop the thread
    m_Thread.Wait(5000);    //Give 5 second to finish execution pending jobs in the thread.

    CFrameWnd::OnDestroy();
}


BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
    if( !CFrameWnd::PreCreateWindow(cs) )
        return FALSE;

    cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
    cs.lpszClass = AfxRegisterWndClass(0);
    return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers
void CMainFrame::OnSetFocus(CWnd* pOldWnd)
{
    // forward focus to the view window
    //m_wndView.SetFocus();
    m_Grid.SetFocus();
}

BOOL CMainFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
    // let the splitter have first crack at the command
    if (m_wndSplitter.OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
        return TRUE;

    // otherwise, do default handling
    return CFrameWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}


//Adding items
void CMainFrame::OnAddItems() 
{
    
    CAddItemsDlg dlg;
    if((dlg.DoModal() == IDOK) && (dlg.GetCount() > 0))
    {
        if(dlg.GetThread() == GUIThread)
        {
            //Add items from the GUI thread
            AddItemsToGrid(dlg.GetCount(), dlg.GetLocation(), GUIThread);
        }
        else
        {
            m_Thread.PushJob(new CAddItemsFromNonGUIThreadJob(*this, dlg.GetCount(), dlg.GetLocation()));
        }
    }
}

//This function can be called from any thread. 
void CMainFrame::AddItemsToGrid(UINT count, Location location, Thread thread)
{
    //measure execution time with the high resolution timer
    LARGE_INTEGER ticksPerSecond;
    LARGE_INTEGER start_ticks;
    LARGE_INTEGER end_ticks;
    BOOL timerAvailable = TRUE;
    timerAvailable &= QueryPerformanceFrequency(&ticksPerSecond);
    timerAvailable &= QueryPerformanceCounter(&start_ticks);
    
    //Reset counters to update items
    {
        Common::CLock lock(m_Synchro);
        m_StartTime = GetTickCount();
        m_UpdatedItems = 0;
    }
    

    int rowsBefore = m_Grid.GetRowCount();
    CString itemName;
    for(UINT i = 0; i < count; ++i)
    {
        //Select insertion position
        GUI::HITEM hItem = GUI::GI_LAST;
        if(location == Head)
        {
            hItem = GUI::GI_FIRST;
        }
        if(location == Random)
        {
            if(rowsBefore + i > 0)
            {
                 hItem = m_Grid.GetItem((rand()*rand())%(rowsBefore + i));
            }
        }

    
        CTestObject* object = 0;

        //Protect m_TestObjects against racing conditions
        {
            Common::CLock lock(m_Synchro);

            //Create a new name for item
            itemName.Format(_T("Item %d"), m_TestObjects.size());
            object = new CTestObject((const TCHAR*)itemName);


            //Add the item to the container
            m_TestObjects.push_back(object);
        }

        //Add the created item to the upper grid without lock
        m_Grid.Add(object, GUI::GI_SCROLLABLE_ROOT, hItem);
    }


    //Compute the time
    timerAvailable &= QueryPerformanceCounter(&end_ticks);

    int rowsAfter = m_Grid.GetRowCount();

    double span = timerAvailable && ticksPerSecond.QuadPart > 0 
        ? double(end_ticks.QuadPart- start_ticks.QuadPart)/ticksPerSecond.QuadPart 
        : 0;
    long avarageSpeed = span > 0 ? long(double(count)/span) : 0;


    CString text;
    text.Format(_T("%d items have been added %sfrom %s thread in %f sec. Items in grid %d => %d. Average speed = %d lines/sec." )
        , count
        , location == Head ? _T("to head ") : location == Tail ? _T("to tail ") : _T("")
        , thread == GUIThread ? _T("GUI") : _T("non-GUI")
        , span
        , rowsBefore
        , rowsAfter
        , avarageSpeed);

    //Add log message to the logger
    LogMessage(AddItem, text);
}


//Removing items
void CMainFrame::OnRemoveItems() 
{
    CRemoveItemsDlg dlg;
	dlg.SetCount(m_Grid.GetRowCount());
	
    if((dlg.DoModal() == IDOK) && (dlg.GetCount() > 0))
    {
        if(dlg.GetThread() == GUIThread)
        {
            //Add items from the GUI thread
            RemoveItemsFromGrid(dlg.GetCount(), dlg.GetLocation(), GUIThread);
        }
        else
        {
            m_Thread.PushJob(new CRemoveItemsFromNonGUIThreadJob(*this, dlg.GetCount(), dlg.GetLocation()));
        }
    }
}


//This function can be called from any thread. 
void CMainFrame::RemoveItemsFromGrid(UINT count, Location location, Thread thread)
{
    if(count > m_Grid.GetRowCount())
    {
        //Add log message to the logger
        LogMessage(RemoveItem, _T("Can't remove requested count of items"));
        return;
    }


    //Measure execution time with the high resolution timer
    LARGE_INTEGER ticksPerSecond;
    LARGE_INTEGER start_ticks;
    LARGE_INTEGER end_ticks;
    BOOL timerAvailable = TRUE;
    timerAvailable &= QueryPerformanceFrequency(&ticksPerSecond);
    timerAvailable &= QueryPerformanceCounter(&start_ticks);
    

    int rowsBefore = m_Grid.GetRowCount();
    CString itemName;
    for(UINT i = 0; i < count; ++i)
    {
        Common::CDataObject* item = 0;

        //This function can be called from any thread. Protect m_TestObjects from racing conditions
        {
            Common::CLock lock(m_Synchro);
            //Remove item from container
            if(!m_TestObjects.empty())
            {
                item = *m_TestObjects.rbegin();
                m_TestObjects.erase(m_TestObjects.end() - 1);
            }
        }

        if(item)
        {
            //Delete item from the grid and destroy it
            m_Grid.Delete(item);
            delete item;
        }
    }


    //Compute the time
    timerAvailable &= QueryPerformanceCounter(&end_ticks);

    int rowsAfter = m_Grid.GetRowCount();

    double span = timerAvailable && ticksPerSecond.QuadPart > 0 
        ? double(end_ticks.QuadPart- start_ticks.QuadPart)/ticksPerSecond.QuadPart 
        : 0;
    long avarageSpeed = span > 0 ? long(double(count)/span) : 0;


    CString text;
    text.Format(_T("%d items have been removed %sfrom %s thread in %f sec. Items in grid %d => %d. Average speed = %d lines/sec." )
        , count
        , location == Head ? _T("from head ") : location == Tail ? _T("from tail ") : _T("")
        , thread == GUIThread ? _T("GUI") : _T("non-GUI")
        , span
        , rowsBefore
        , rowsAfter
        , avarageSpeed);

    //Add log message to the logger
    LogMessage(RemoveItem, text);
}

//Update settings dialog
void CMainFrame::OnUpdateSettings() 
{
    CUpdateItemsDlg dlg(m_Frequency, m_UpdatesSource);
    if(dlg.DoModal() == IDOK)
    {
        Common::CLock lock(m_Synchro);
        m_UpdatesSource = dlg.GetSource();
        m_Frequency = dlg.GetFrequency();
        m_StartTime = GetTickCount();
        m_UpdatedItems = 0;

        CString message;
        message.Format(_T("Updates from %s thead with frequency %d upd/sec. %s"),
                        m_UpdatesSource == NonGUIThread ? _T("non-GUI") : _T("GUI"),
                        m_Frequency,
                        m_EnableUpdates ? _T("") : _T("Updates are actually disabled."));
        
        LogMessage(ChangeSettings, message);
    }
}




//Enable/disable updates
void CMainFrame::OnEnableUpdates() 
{
    Common::CLock lock(m_Synchro);

    m_EnableUpdates = !m_EnableUpdates;
    m_StartTime = GetTickCount();
    m_UpdatedItems = 0;

    CString message;
    message.Format(_T("Updates are %s."), m_EnableUpdates ? _T("enabled") : _T("disabled"));
    LogMessage(ChangeSettings, message);
}

void CMainFrame::OnUpdateEnableUpdates(CCmdUI* pCmdUI) 
{
    Common::CLock lock(m_Synchro);

    pCmdUI->SetCheck(m_EnableUpdates);
    
}

//Non-GUI timer event handler
void CMainFrame::OnExpire(int timerId)
{
    //This function is called in the non-GUI thread
    ProcessUpdate(NonGUIThread);

    //Restart non-GUI timer
    Common::CTimer::StartTimer(this, TimerId, 10); 
}

//GUI timer event handler
void CMainFrame::OnTimer(UINT nIDEvent) 
{
    if(nIDEvent == TimerId)
    {
        //This function is called in the GUI thread
        ProcessUpdate(GUIThread);
    }
    if(nIDEvent == TitleUpdateTimerId)
    {
        UpdateTitle();
    }


    CFrameWnd::OnTimer(nIDEvent);
}


//Updating data in the grid (can be called from any thread)
void CMainFrame::ProcessUpdate(Thread thread)
{
    Common::CLock lock(m_Synchro);

    if(m_EnableUpdates && (m_Frequency > 0) && (m_UpdatesSource == thread) && !m_TestObjects.empty())
    {
        long curTime = GetTickCount();
        long interval = curTime - m_StartTime;

        //Check for the int32 overflow, returned by the GetTickCount(). 
        if(m_StartTime > curTime)
        {
            Common::CLock lock(m_Synchro);
            m_UpdatedItems = 0;
            m_StartTime = curTime;
        }
        if(interval > 0)
        {
            //Evaluate random item
            __int64 expectedUpdates = __int64(interval) * __int64(m_Frequency) / 1000;
            for(__int64 i = m_UpdatedItems; i < expectedUpdates; ++i)
            {
                Container::size_type index = ((rand()*rand())%m_TestObjects.size());
                CTestObject* object = m_TestObjects[index];
                object->EvaluateRandomField();
            
                m_UpdatedItems++;
            }
        }
    }
}







//Enable/disable grid optimizations
void CMainFrame::OnOnidleOptimization() 
{
    theApp.SetOnIdleOptimization(!theApp.GetOnIdleOptimization());

    CString message;
    message.Format(_T("OnIdle optimization is %s."), theApp.GetOnIdleOptimization() ? _T("enabled") : _T("disabled"));
    LogMessage(ChangeSettings, message);

}

void CMainFrame::OnUpdateOnidleOptimization(CCmdUI* pCmdUI) 
{
    pCmdUI->SetCheck(theApp.GetOnIdleOptimization() ? 1 : 0);
}

void CMainFrame::OnNotificationsOptimization() 
{
    m_Grid.AllowNotifications(!m_Grid.IsNotificationsAllowed());

    CString message;
    message.Format(m_Grid.IsNotificationsAllowed() 
                    ? _T("Allow grid to send notifications to the parent.") 
                    : _T("Prevent grid from sending notifications"));
    LogMessage(ChangeSettings, message);

}

void CMainFrame::OnUpdateNotificationsOptimization(CCmdUI* pCmdUI) 
{
    pCmdUI->SetCheck(m_Grid.IsNotificationsAllowed() ? 0 : 1);
}

//Double buffering 
void CMainFrame::OnDoubleBuffering() 
{
    m_Grid.SetFlickerFreeDrawing(!m_Grid.GetFlickerFreeDrawing());

    CString message;
    message.Format(_T("Double buffering is %s."), m_Grid.GetFlickerFreeDrawing() ? _T("enabled") : _T("disabled"));
    LogMessage(ChangeSettings, message);
}

void CMainFrame::OnUpdateDoubleBuffering(CCmdUI* pCmdUI) 
{

    pCmdUI->SetCheck(m_Grid.GetFlickerFreeDrawing() ? 1 : 0);
}



//Highlighting modes
void CMainFrame::OnHighlightDisable() 
{
    m_Grid.AllowHighlight(false);
    LogMessage(ChangeSettings, _T("Highlighting is disabled"));
}

void CMainFrame::OnUpdateHighlightDisable(CCmdUI* pCmdUI) 
{
    pCmdUI->SetCheck(m_Grid.CanHighlight() ? 0 : 1);
    
}

void CMainFrame::OnHighlightAlpha() 
{
    m_Grid.GetHighlightSettings().SetMode(GUI::CHighlightSettings::AlphaBlendBackColor);
    m_Grid.AllowHighlight(true);

    LogMessage(ChangeSettings, _T("Alpha-blend highlighting is enabled"));

}

void CMainFrame::OnUpdateHighlightAlpha(CCmdUI* pCmdUI) 
{
    pCmdUI->SetCheck(m_Grid.CanHighlight() && 
                    (m_Grid.GetHighlightSettings().GetMode() == GUI::CHighlightSettings::AlphaBlendBackColor));

}

void CMainFrame::OnHighlightFading() 
{
    m_Grid.GetHighlightSettings().SetMode(GUI::CHighlightSettings::FadingBackColor);
    m_Grid.AllowHighlight(true);

    LogMessage(ChangeSettings, _T("Fading effect is enabled"));
}

void CMainFrame::OnUpdateHighlightFading(CCmdUI* pCmdUI) 
{
    pCmdUI->SetCheck(m_Grid.CanHighlight() && 
                    (m_Grid.GetHighlightSettings().GetMode() == GUI::CHighlightSettings::FadingBackColor));

}



//Logging
void CMainFrame::LogMessage(LogActionType actionType, const CString& message)
{
    CLogMessage* logMessage = new CLogMessage(actionType, (const TCHAR*)message);

    //Protect m_LogMessages against racing conditions
    {
        Common::CLock lock(m_Synchro);
        m_LogMessages.push_back(logMessage);
    }
    m_Logger.Add(logMessage);
}

void CMainFrame::UpdateTitle()
{
    //Compute CPU Usage
    long interval = GetTickCount() - m_LastCPUUsageTick;
    interval = interval < 0 ? -interval : interval;

    FILETIME startTime;
    FILETIME exitTime;
    union {FILETIME ftTime; __int64  int64Time; } kernel; // time the process has spent in kernel mode
    union {FILETIME ftTime; __int64  int64Time; } user;   // time the process has spent in user mode
    ::GetProcessTimes(::GetCurrentProcess(), &startTime, &exitTime, &kernel.ftTime, &user.ftTime);

    // Process times are returned in a 64-bit structure, as the number of 100 nanosecond ticks 
    // since 1 January 1601. User mode and kernel mode times for this process are in separate 64-bit 
    // structures. Now, we will compute sum of high 32-bit quantities and convert the result to seconds.
    double val = double(kernel.int64Time + user.int64Time)*(1.0e-7);

    double usage = interval > 0 ? 1000*(val - m_LastConsumedTime)/double(interval) : 0;
    m_LastConsumedTime = val;
    m_LastCPUUsageTick = GetTickCount();

    CSize gridSize(0, 0);
    if(m_Grid.GetSafeHwnd())
    {
        WINDOWPLACEMENT wp;
        m_Grid.GetWindowPlacement(&wp);
        
        gridSize = CSize(wp.rcNormalPosition.right - wp.rcNormalPosition.left, 
                         wp.rcNormalPosition.bottom - wp.rcNormalPosition.top);
    }

    CString text;
    text.Format(_T("Dapfor grid: items = %d, size = %dx%d, CPU usage = %.2f%%"), 
                m_Grid.GetRowCount(),
                gridSize.cx,
                gridSize.cy,
                100*usage);

    SetWindowText(text);
}
