// FileManager.h: interface for the CFileManager class.
//
//////////////////////////////////////////////////////////////////////

#pragma once

#include <Dapfor/Common/Automat.h>
#include <Dapfor/Common/SyncObject.h>
#include <vector>

class CFolderData;
class CFileData;

//////////////////////////////////////////////////////////////////////////
//The manager looks for the drives, folders and files in its own thread 
//to avoid freezes of the GUI interface.
//////////////////////////////////////////////////////////////////////////
class CFileManager : public Dapfor::Common::CThread
{
public:
    //This interface permits to get notifications from this manager.
    //Note that notifications are raised in the non-GUI thread.
    struct IListener
    {
        //Called when a new drive is found.
        virtual void OnAddDrive(CFolderData& folder) = 0;
        
        //Called when a new folder is found.
        virtual void OnAddFolder(CFolderData& folder) = 0;

        //Called when a new file is found.
        virtual void OnAddFile(CFileData& file) = 0;
    };

    //Type definitions  
    typedef std::vector<CFolderData*> Disks;
    typedef std::vector<IListener*> Listeners;


public:
    //Constructor and destructor
    CFileManager();
    virtual ~CFileManager();

    //Starts the manager in the non-GUI thread
    virtual int Run();


    void BuildDrives();                     //Looks for hard drives
    void BindFolders(CFolderData* root);    //Builds recursively an hierarchy of folders


    //Subscription for events
    void AddListener(IListener* listener);
    void RemoveListener(IListener* listener);

private:
    Disks       m_Disks;
    Listeners   m_Listeners;
    
    //The critical section that protects m_Listeners & m_Disks variables against race conditions
    Dapfor::Common::CSyncObject m_Synchro; 
};


