/////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright 2007-2008 Dapfor
//                                                                         
// This file is part of the Dapfor software, and may only be used, modified, and distributed 
// under the terms of the Dapfor Software License, Version 1.0.
// (See accompanying file lisense.txt or open the license at www.dapfor.com/license_agreement.aspx)
// By continuing to use, modify, or distribute this file you indicate that you have read the license 
// and understand and accept it fully.
/////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once 

#include <Dapfor/GUI/Types.h>
#include <Dapfor/GUI/GUI_Export.h>

namespace Dapfor
{
namespace GUI
{

//////////////////////////////////////////////////////////////////////////
/// \brief IForEach interface.
///
/// IForEach interface is designed for performing operations over inserted into the grid data objects. 
/// According to the assigned criteria the grid finds all corresponding handles and forward them 
/// to the () operator. 
///
/// \par ThreadSafety:
/// The operator () will be always called in the %GUI thread.
///
/// \version <b>2.0.0</b> Initial version
///
/// \class  IForEach IForEach.h Dapfor/GUI/IForEach.h
/// \file IForEach.h
//////////////////////////////////////////////////////////////////////////
class DF_GUI_API IForEach
{
public:
    /// Called to perform some operations on a handle and that corresponds to the data object, 
    /// inserted in CGrid
    /// \param[in] grid Reference to CGrid object
    /// \param[in] item Handle associated with the data object, inserted in CGrid.
    /// \param[in] pDO Pointer to a data object.
    /// \return False if no more iterations are needed. Otherwise true.
    virtual bool operator () (CGrid& grid, HITEM item, Common::CDataObject* pDO) = 0;
};

}//namespace GUI

}//namespace Dapfor

