/////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright 2007-2008 Dapfor
//                                                                         
// This file is part of the Dapfor software, and may only be used, modified, and distributed 
// under the terms of the Dapfor Software License, Version 1.0.
// (See accompanying file lisense.txt or open the license at www.dapfor.com/license_agreement.aspx)
// By continuing to use, modify, or distribute this file you indicate that you have read the license 
// and understand and accept it fully.
/////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include <Dapfor/GUI/GUI_Export.h>
#include <Dapfor/GUI/Types.h>
#include <Dapfor/GUI/IClipboardFormat.h>

namespace Dapfor
{
namespace GUI
{

class CHeader;

//////////////////////////////////////////////////////////////////////////
///\brief Text data source for clipboard copy and drag&prop operations.
///
/// \version <b>1.0.0.0</b> Initial version
///
/// \class  CTextClipboardFormat TextClipboardFormat.h Dapfor/GUI/TextClipboardFormat.h
//////////////////////////////////////////////////////////////////////////
class DF_GUI_API CTextClipboardFormat : public IClipboardFormat
{
public:
    /// Constructor 
    CTextClipboardFormat();

    /// Virtual destructor
    ~CTextClipboardFormat();

    // Prepares data for serialization to the clipboard.
    virtual void Prepare(const Handles& handles, const CHeader& header);

    // Gets the size in bytes needed to store data to the binary buffer
    virtual long GetSerializationSize() const;
    
    // Serializes data to the binary buffer
    virtual void Serialize(void* buffer, const CGrid& grid, ContextType contextType, long userData) const;

    // Clears the temporary data, created during a serialization process.
    virtual void Clear();

    /// Gets CF_TEXT identifier.
    virtual int  GetFormatId() const;

    // Deserializes data from the clipboard and tries to restore grid handles with its hierarchy.
    virtual bool Deserialize(const void* buffer, 
                             long bufSize, 
                             const IObjectImporter* importer, 
                             IClipboardFormat::RestoredInfo& restoredInfo);


private:
    CTextClipboardFormat(const CTextClipboardFormat&);
    CTextClipboardFormat& operator = (const CTextClipboardFormat&);

    struct Impl;
    Impl* m_Impl;
};

}

}//namespace Dapfor

